package com.logrocket.plugins.react;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build.VERSION;
import android.os.Build.VERSION_CODES;
import android.view.View;
import android.webkit.WebView;
import androidx.annotation.Keep;
import androidx.annotation.NonNull;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.ReadableMapKeySetIterator;
import com.facebook.react.bridge.ReadableType;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.facebook.react.uimanager.util.ReactFindViewUtil;
import com.facebook.react.uimanager.util.ReactFindViewUtil.OnViewFoundListener;
import com.logrocket.core.CustomEventBuilder;
import com.logrocket.core.IdentifyHelper;
import com.logrocket.core.LogRocketCore;
import com.logrocket.core.SDK;
import com.logrocket.core.SDK.OptionsConfiguration;
import com.logrocket.core.SDK.SanitizerType;
import com.logrocket.core.TelemetryReporter;
import com.logrocket.core.util.logging.Logger;
import com.logrocket.core.util.logging.TaggedLogger;
import com.logrocket.core.MessageReceiverHelper;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class LogRocketNativeModule extends ReactContextBaseJavaModule {

  private final ReactApplicationContext reactContext;
  private final Logger logger = new TaggedLogger("react-native");

  private static final String SDK_ERROR_ACTION = "LogRocketSDK.Error";
  private static final String SDK_SESSION_ACCEPTED_ACTION = "LogRocketSDK.SessionAccepted";
  private static final String SDK_SEND_REDUX_INITIAL_ACTION = "LogRocketSDK.StartSessionFromInactive";
  private static String currentAppId = "";
  private List<JSONArray> messageList = new ArrayList<>();

  public LogRocketNativeModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
    LogRocketReceiver receiver = new LogRocketReceiver(this.reactContext);

    IntentFilter filter = new IntentFilter();
    filter.addAction(SDK_ERROR_ACTION);
    filter.addAction(SDK_SESSION_ACCEPTED_ACTION);
    filter.addAction(SDK_SEND_REDUX_INITIAL_ACTION);

    if (VERSION.SDK_INT >= VERSION_CODES.TIRAMISU) {
      this.reactContext.registerReceiver(receiver, filter, Context.RECEIVER_NOT_EXPORTED);
    } else {
      this.reactContext.registerReceiver(receiver, filter);
    }
  }

  private static Map<String, String> formatReadableMap(ReadableMap map) {
    Map<String, String> formattedMap = new HashMap<>();

    if (map == null) {
      return formattedMap;
    }

    Iterator<Map.Entry<String, Object>> iterator = map.getEntryIterator();

    while (iterator.hasNext()) {
      Map.Entry<String, Object> entry = iterator.next();
      formattedMap.put(entry.getKey(), entry.getValue().toString());
    }

    return formattedMap;
  }

  @NonNull
  @Override
  public String getName() {
    return "LogRocketNativeModule";
  }

  @ReactMethod
  public void addListener(String eventName) {}

  @ReactMethod
  public void removeListeners(Integer count) {}

  @Keep
  @ReactMethod
  public void initWithConfig(String appID, ReadableMap config, Promise promise) {
    try {
      currentAppId = appID;
      promise.resolve(
          SDK.init(
              this.reactContext,
              createOptionsConfiguration(appID, config)));
    } catch (Throwable th) {
      this.logger.error("Error during init", th);
      TelemetryReporter.sendErrorTelemetry(th);
      promise.resolve(false);
    }
  }

  @Keep
  @ReactMethod
  public void pauseViewCapture() {
    try {
      SDK.pauseViewCapture();
    } catch (Throwable th) {
      this.logger.error("Error during pauseViewCapture call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void unpauseViewCapture() {
    try {
      SDK.unpauseViewCapture();
    } catch (Throwable th) {
      this.logger.error("Error during unpauseViewCapture call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void installShutdownHandler(Callback callback) {
    try {
      SDK.onShutdown(callback::invoke);
    } catch (Throwable th) {
      this.logger.error("Error during installShutdownHandler call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }


  @Keep
  @ReactMethod
  public void identifyWithTraits(String uid, ReadableMap traits, Boolean isAnonymous) {
    try {
      IdentifyHelper.identify(uid, formatReadableMap(traits), isAnonymous);
    } catch (Throwable th) {
      this.logger.error("Error during identify call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void shutdown() {
    try {
      SDK.endSession();
    } catch (Throwable th) {
      this.logger.error("Error while shutting down", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void track(String customEventName, ReadableMap eventProperties) {
    try {
      CustomEventBuilder builder = new CustomEventBuilder(customEventName);

      Iterator<Map.Entry<String, Object>> iterator = eventProperties.getEntryIterator();
      while (iterator.hasNext()) {
        Map.Entry<String, Object> entry = iterator.next();
        String keyString = entry.getKey().toString();
        ReadableMap protobufVariant = (ReadableMap) entry.getValue();

        if (protobufVariant.hasKey("doubleVal")) {
          ReadableArray readableArray = protobufVariant.getArray("doubleVal");
          if (readableArray != null) {
            Double[] list = new Double[readableArray.size()];
            for (int i = 0; i < readableArray.size(); i++) {
              list[i] = readableArray.getDouble(i);
            }

            builder.put(keyString, list);
          }
        } else if (protobufVariant.hasKey("boolVal")) {
          ReadableArray readableArray = protobufVariant.getArray("boolVal");
          if (readableArray != null) {
            Boolean[] list = new Boolean[readableArray.size()];
            for (int i = 0; i < readableArray.size(); i++) {
              list[i] = readableArray.getBoolean(i);
            }

            builder.put(keyString, list);
          }
        } else if (protobufVariant.hasKey("stringVal")) {
          ReadableArray readableArray = protobufVariant.getArray("stringVal");
          if (readableArray != null) {
            String[] list = new String[readableArray.size()];
            for (int i = 0; i < readableArray.size(); i++) {
              list[i] = readableArray.getString(i);
            }

            builder.put(keyString, list);
          }
        }
      }
      SDK.track(builder);
    } catch (Throwable th) {
      this.logger.error("Error during track call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void tagPage(String relativePage) {
    try {
      SDK.tagPage(relativePage);
    } catch (Throwable th) {
      this.logger.error("Error during tagPage call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void registerWebView(String viewId) {
    try {
      View rootView = Objects.requireNonNull(reactContext.getCurrentActivity())
        .getWindow()
        .getDecorView()
        .getRootView();

      // The RN view hierarchy may not be finished loading by the time this code is run,
      //  so we pass a callback to do the viewId comparison and register the webview once
      //  it's found (see here: https://github.com/facebook/react-native/blob/main/packages/react-native/ReactAndroid/src/main/java/com/facebook/react/uimanager/util/ReactFindViewUtil.java#L74-L76)
      ReactFindViewUtil.findView(rootView, new OnViewFoundListener() {
        @Override
        public String getNativeId() {
          return viewId;
        }

        @Override
        public void onViewFound(View webView) {
          if (webView instanceof WebView) {
            SDK.registerWebView((WebView) webView);
            return;
          } else if (webView.getClass().getName().equals("com.reactnativecommunity.webview.RNCWebViewWrapper")) {
            // Starting with v13.6.2, react-native-webview adds the actual WebView as the child of a wrapper class:
            View childView = ((android.widget.FrameLayout)webView).getChildAt(0);
            if (childView instanceof WebView) {
              SDK.registerWebView((WebView) childView);
              return;
            }
          }
          String msg = String.format("Unable to find WebView for ID '%s'", viewId);
          logger.error(msg);
          LogRocketCore.debugLog(msg);
        }
      });
    } catch (Throwable th) {
      this.logger.error("Error during registerWebView call", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void postMessage(ReadableArray message){
    try {
      JSONArray array = toJSONArray(message);
      messageList.add(array);
    } catch (Throwable th) {
      this.logger.error("Error while capturing message", th);
      TelemetryReporter.sendErrorTelemetry(th);
    }
  }

  @Keep
  @ReactMethod
  public void donePostingMessage() {
    MessageReceiverHelper.rebuildMessage(messageList);
    messageList = new ArrayList<>();
  }

  static class LogRocketReceiver extends BroadcastReceiver {
    private final ReactApplicationContext reactContext;
    private final Logger logger = new TaggedLogger("logrocket-receiver");

    public LogRocketReceiver(ReactApplicationContext reactContext) {
      this.reactContext = reactContext;
    }

    @Override
    public void onReceive(Context context, Intent intent) {
      String action = intent.getAction();
      try {
        if (SDK_ERROR_ACTION.equals(action)) {
          String errorMessage = intent.getStringExtra("errorMessage");
          boolean shouldDisableAndFreeMemory =
              intent.getBooleanExtra("shouldDisableAndFreeMemory", false);
          boolean shouldWarnAboutReduxSize =
              intent.getBooleanExtra("shouldWarnAboutReduxSize", false);
          WritableMap params = Arguments.createMap();
          if (errorMessage != null && errorMessage.length() > 0) {
            params.putString("errorMessage", errorMessage);
          }

          params.putBoolean("shouldDisableAndFreeMemory", shouldDisableAndFreeMemory);
          params.putBoolean("shouldWarnAboutReduxSize", shouldWarnAboutReduxSize);

          this.reactContext
              .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
              .emit("LogRocketSDKOnError", params);
        } else if (SDK_SESSION_ACCEPTED_ACTION.equals(action)) {
          String appID = intent.getStringExtra("appID");
          if (!currentAppId.equals(appID)) {
            // Ignore LogRocket session acceptance from other apps
            return;
          }
          String sessionURL = intent.getStringExtra("sessionURL");
          WritableMap params = Arguments.createMap();
          params.putString("sessionURL", sessionURL);

          this.reactContext
              .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
              .emit("LogRocketSDKOnSessionAccepted", params);
        } else if (SDK_SEND_REDUX_INITIAL_ACTION.equals(action)) {
          String appID = intent.getStringExtra("appID");
          if (!currentAppId.equals(appID)) {
            // Ignore messages from other apps
            return;
          }

          this.reactContext
              .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
              .emit("LogRocketSDKOnStartSessionFromInactive", Arguments.createMap());
        }
      } catch (Throwable th) {
        this.logger.error("Error while sending " + action + " event from Native Module", th);
        TelemetryReporter.sendErrorTelemetry(th);
      }
    }
  }

  private OptionsConfiguration createOptionsConfiguration(String appID, ReadableMap config) {
    return options -> {
      options.setAppID(appID);
      options.setIsReactNative(true);
      options.setIsRNNewArchEnabled(BuildConfig.RN_NEW_ARCH_ENABLED);

      if (config == null) {
        return;
      }

      if (config.hasKey("serverURL")) {
        options.setServerURL(config.getString("serverURL"));
      }

      if (config.hasKey("updateId")) {
        options.setUpdateId(config.getString("updateId"));
      }

      if (config.hasKey("expoChannel")) {
        options.setExpoChannel(config.getString("expoChannel"));
      }

      if (config.hasKey("enableIPCapture")) {
        options.setEnableIPCapture(config.getBoolean("enableIPCapture"));
      }

      if (config.hasKey("enableJetpackCompose")) {
        options.setEnableJetpackCompose(config.getBoolean("enableJetpackCompose"));
      }

      if (config.hasKey("forceCleanStart")) {
        options.setForceCleanStart(config.getBoolean("forceCleanStart"));
      }

      if (config.hasKey("uploadIntervalMs")) {
        options.setUploadIntervalMs(config.getInt("uploadIntervalMs"));
      }

      if (config.hasKey("viewScanIntervalSeconds")) {
        options.setViewScanIntervalSeconds(config.getInt("viewScanIntervalSeconds"));
      }

      if (config.hasKey("captureRedactedViewTouches")) {
        options.setCaptureRedactedViewTouches(config.getBoolean("captureRedactedViewTouches"));
      }

      if (config.hasKey("viewScanningEnabled")) {
        options.setEnableViewScanning(config.getBoolean("viewScanningEnabled"));
      }

      if (config.hasKey("logLevel") && !config.isNull("logLevel")) {
        //noinspection ConstantConditions
        options.setLogLevel(SDK.LogLevel.fromString(config.getString("logLevel")));
      }

      if (config.hasKey("connectionType") && !config.isNull("connectionType")) {
        //noinspection ConstantConditions
        options.setConnectionType(
                SDK.ConnectionType.fromString(config.getString("connectionType")));
      }

      if (config.hasKey("allowTags")) {
        ReadableArray allowTags = config.getArray("allowTags");
        if (allowTags != null) {
          ArrayList<Object> tags = allowTags.toArrayList();
          for (Object tag : tags) {
            options.addAllowTag(tag);
          }
        }
      }

      if (config.hasKey("redactionTags")) {
        ReadableArray redactionTags = config.getArray("redactionTags");
        if (redactionTags != null) {
          ArrayList<Object> tags = redactionTags.toArrayList();
          for (Object tag : tags) {
            options.addRedactionTag(tag);
          }
        }
      }

      if (config.hasKey("enablePersistence")) {
        options.setEnablePersistence(config.getBoolean("enablePersistence"));
      }

      if (config.hasKey("shouldDetectExceptions")) {
        options.setShouldDetectExceptions(config.getBoolean("shouldDetectExceptions"));
      }

      if (config.hasKey("textSanitizer") && !config.isNull("textSanitizer")) {
        //noinspection ConstantConditions
        options.setTextSanitizer(
                SanitizerType.fromString(config.getString("textSanitizer")));
      }

      ReadableMap androidProxy = config.getMap("androidProxy");
      if (androidProxy != null) {
        options.setProxyHost(androidProxy.getString("host"));
        options.setProxyPort(androidProxy.getInt("port"));
        options.setProxyAuthHeaderName(androidProxy.getString("authHeaderName"));
        options.setProxyAuthHeaderValue(androidProxy.getString("authHeaderValue"));
      }

      if (config.hasKey("enableAutomaticLifecycleCapture")) {
        options.setEnableAutomaticLifecycleCapture(config.getBoolean("enableAutomaticLifecycleCapture"));
      }

      if (config.hasKey("viewCaptureTimeoutThreshold")) {
        options.setViewCaptureTimeoutThreshold(config.getInt("viewCaptureTimeoutThreshold"));
      }

      if (config.hasKey("viewCaptureAdditionalWireframeDuration")) {
        options.setViewCaptureAdditionalWireframeDuration(config.getInt("viewCaptureAdditionalWireframeDuration"));
      }
    };
  }

  // helpers found here: https://gist.github.com/mfmendiola/bb8397162df9f76681325ab9f705748b
  /*
  MIT License
  Copyright (c) 2020 Marc Mendiola
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
 */

  private static JSONObject toJSONObject(ReadableMap readableMap) throws JSONException {
    JSONObject jsonObject = new JSONObject();

    ReadableMapKeySetIterator iterator = readableMap.keySetIterator();

    while (iterator.hasNextKey()) {
      String key = iterator.nextKey();
      ReadableType type = readableMap.getType(key);

      switch (type) {
        case Null:
          // The standard Java null reference causes JSONArray.get to fail so we
          // use JSONObject.NULL instead.
          // reference: https://developer.android.com/reference/org/json/JSONArray
          jsonObject.put(key, JSONObject.NULL);
          break;
        case Boolean:
          jsonObject.put(key, readableMap.getBoolean(key));
          break;
        case Number:
          jsonObject.put(key, readableMap.getDouble(key));
          break;
        case String:
          String string = readableMap.getString(key);
          if(string != null) {
            jsonObject.put(key, string);
          }
          break;
        case Map:
          ReadableMap map = readableMap.getMap(key);
          if(map != null) {
            jsonObject.put(key, toJSONObject(map));
          }
          break;
        case Array:
          ReadableArray array = readableMap.getArray(key);
          if(array != null) {
            jsonObject.put(key, toJSONArray(array));
          }
          break;
      }
    }

    return jsonObject;
  }

  private static JSONArray toJSONArray(ReadableArray readableArray) throws JSONException {
    JSONArray jsonArray = new JSONArray();

    for (int i = 0; i < readableArray.size(); i++) {
      ReadableType type = readableArray.getType(i);

      switch (type) {
        case Null:
          // The standard Java null reference causes JSONArray.get to fail so we
          // use JSONObject.NULL instead.
          // reference: https://developer.android.com/reference/org/json/JSONArray
          jsonArray.put(i, JSONObject.NULL);
          break;
        case Boolean:
          jsonArray.put(i, readableArray.getBoolean(i));
          break;
        case Number:
          jsonArray.put(i, readableArray.getDouble(i));
          break;
        case String:
          jsonArray.put(i, readableArray.getString(i));
          break;
        case Map:
          jsonArray.put(i, toJSONObject(readableArray.getMap(i)));
          break;
        case Array:
          jsonArray.put(i, toJSONArray(readableArray.getArray(i)));
          break;
      }
    }

    return jsonArray;
  }
}
