const {
  withProjectBuildGradle,
  createRunOncePlugin,
  withAppBuildGradle,
  withXcodeProject,
} = require('@expo/config-plugins');
const {
  getXCConfigurationListEntries,
  getBuildConfigurationsForListId,
} = require('@expo/config-plugins/build/ios/utils/Xcodeproj');
const {
  removeGeneratedContents,
  createGeneratedHeaderComment,
} = require('@expo/config-plugins/build/utils/generateCode');
const { withBuildProperties } = require('expo-build-properties');
const fs = require('fs');
const path = require('path');

const LOGROCKET_MAVEN_URL = 'maven { url "https://storage.googleapis.com/logrocket-maven/" }';
const LOGROCKET_DEPENDENCY_CLASSPATH = 'classpath "com.logrocket.gradle:android:0.3.+"';
const LOGROCKET_PLUGIN = 'apply plugin: \'com.logrocket.gradle.android\'';

const buildscriptRepositoriesRegex = new RegExp(/buildscript\s*{.*?repositories\s*{.*?(?=\n\s*})/, 'sg');
const buildscriptDependenciesRegex = new RegExp(/buildscript\s*{.*?dependencies\s*{.*?(?=\n\s*})/, 'sg');

function createGeneratedHeaderAndFooter(tag) {
  return {
    header: createGeneratedHeaderComment(tag, tag, '//'),
    footer: `// @generated end ${tag}`,
  };
}

function modProjectGradle(buildGradle) {
  let modifiedGradle = buildGradle;

  // insert repository
  removeGeneratedContents(modifiedGradle, 'logrocket-buildscript-repositories');
  if (buildscriptRepositoriesRegex.exec(modifiedGradle) !== null) {
    const { header, footer } = createGeneratedHeaderAndFooter('logrocket-buildscript-repositories');
    modifiedGradle = `
      ${modifiedGradle.slice(0, buildscriptRepositoriesRegex.lastIndex)}
      ${header}
      ${LOGROCKET_MAVEN_URL}
      ${footer}
      ${modifiedGradle.slice(buildscriptRepositoriesRegex.lastIndex)}`;
  }

  // insert dependency
  removeGeneratedContents(modifiedGradle, 'logrocket-buildscript-dependencies');
  if (buildscriptDependenciesRegex.exec(modifiedGradle) !== null) {
    const { header, footer } = createGeneratedHeaderAndFooter('logrocket-buildscript-dependencies');
    modifiedGradle = `
      ${modifiedGradle.slice(0, buildscriptDependenciesRegex.lastIndex)}
      ${header}
      ${LOGROCKET_DEPENDENCY_CLASSPATH}
      ${footer}
      ${modifiedGradle.slice(buildscriptDependenciesRegex.lastIndex)}`;
  }

  return modifiedGradle;
}

function modApplicationGradle(buildGradle, pluginProps) {
  let modifiedGradle = buildGradle;

  // insert plugin
  removeGeneratedContents(modifiedGradle, 'logrocket-plugin');
  const comments = createGeneratedHeaderAndFooter('logrocket-plugin');
  modifiedGradle += `
    ${comments.header}
    ${LOGROCKET_PLUGIN}
    ${comments.footer}
    `;

  // insert logrocket config if any plugin props were passed in
  removeGeneratedContents(modifiedGradle, 'logrocket-config');
  if (pluginProps.android.assetCapture) {
    const { header, footer } = createGeneratedHeaderAndFooter('logrocket-config');
    const assetCapture = pluginProps.android.assetCapture;
    let configSection = 'logrocket {\n';
    if (assetCapture.apiKey) {
      configSection += `\tapiKey = "${assetCapture.apiKey}"\n`;
    }
    if (assetCapture.apiHost) {
      configSection += `\tapiHost = "${assetCapture.apiHost}"\n`;
    }
    if (assetCapture.enabledVariants && Array.isArray(assetCapture.enabledVariants)) {
      const enabledVariants = assetCapture.enabledVariants;
      configSection += `\tenabledVariants = [${enabledVariants.map(e => `"${e}"`).join(', ')}].toSet()\n`;
    }
    configSection += '}';
    modifiedGradle = `
      ${modifiedGradle}
      ${header}
      ${configSection}
      ${footer}
      `;
  }

  return modifiedGradle;
}

const withAndroidLogRocket = (config, pluginProps) => {
  // eslint-disable-next-line no-param-reassign
  config = withProjectBuildGradle(config, gradleConfig => {
    if (gradleConfig.modResults.language === 'groovy') {
      // eslint-disable-next-line no-param-reassign
      gradleConfig.modResults.contents = modProjectGradle(gradleConfig.modResults.contents);
    } else {
      throw new Error(
        'Cannot add LogRocket gradle config because the project build.gradle is not groovy'
      );
    }
    return gradleConfig;
  });

  // eslint-disable-next-line no-param-reassign
  config = withAppBuildGradle(config, gradleConfig => {
    if (gradleConfig.modResults.language === 'groovy') {
    // eslint-disable-next-line no-param-reassign
      gradleConfig.modResults.contents = modApplicationGradle(gradleConfig.modResults.contents, pluginProps);
    } else {
      throw new Error(
        'Cannot add LogRocket gradle config because the application build.gradle is not groovy'
      );
    }
    return gradleConfig;
  });

  return config;
};

const withIosLogRocket = (config, pluginProps) => {
  // eslint-disable-next-line no-param-reassign
  config = withXcodeProject(config, xcodeProjConfig => {
    const xcodeProject = xcodeProjConfig.modResults;
    const LOGROCKET_UPLOAD_SCRIPT = fs.readFileSync(
      path.resolve(__dirname, '../scripts/asset-upload-script.rb'), 'utf-8');

    // add run script build phase to run ruby script
    const logRocketBuildPhase = xcodeProject.pbxItemByComment(
      'LogRocket Custom Font Capture',
      'PBXShellScriptBuildPhase'
    );
    if (!logRocketBuildPhase) {
      xcodeProject.addBuildPhase([], 'PBXShellScriptBuildPhase', 'LogRocket Custom Font Capture', null, {
        shellPath: '"/usr/bin/env ruby"',
        shellScript: LOGROCKET_UPLOAD_SCRIPT,
      });
    } else {
      logRocketBuildPhase.shellPath = '"/usr/bin/env ruby"';
      logRocketBuildPhase.shellScript = JSON.stringify(LOGROCKET_UPLOAD_SCRIPT);
    }

    // configure environment variables
    getXCConfigurationListEntries(xcodeProject).forEach(configList => {
      getBuildConfigurationsForListId(xcodeProject, configList[0]).forEach(configSection => {
        const buildConfig = configSection[1];
        if (pluginProps.ios.assetCapture) {
          const assetCapture = pluginProps.ios.assetCapture;
          if (assetCapture.apiKey) {
            buildConfig.buildSettings.LOGROCKET_API_KEY = `"${assetCapture.apiKey}"`;
          }
          if (assetCapture.apiHost) {
            buildConfig.buildSettings.LOGROCKET_API_HOST = `"${assetCapture.apiHost}"`;
          }
          if (assetCapture.assetDirectory) {
            buildConfig.buildSettings.LOGROCKET_ASSET_DIR = `"${assetCapture.assetDirectory}"`;
          }
          if (assetCapture.cliPath) {
            buildConfig.buildSettings.LOGROCKET_CLI_PATH = `"${assetCapture.cliPath}"`;
          }
          if (assetCapture.enabledBuildConfigs && Array.isArray(assetCapture.enabledBuildConfigs)) {
            const enabledBuildConfigs = assetCapture.enabledBuildConfigs;
            buildConfig.buildSettings.LOGROCKET_ENABLED_BUILD_CONFIGS = `"${enabledBuildConfigs.join(',')}"`;
          }
        }
      });
    });

    return xcodeProjConfig;
  });
  return config;
};

/*
  pluginProps schema:
  {
    "android": {
      "assetCapture": {
        "apiKey": "apphub:android:123abc", REQUIRED
        "apiHost": "http://localhost:8000",
        "enabledVariants": ["debug", "release"]
      }
    },
    "ios": {
      "assetCapture": {
        "apiKey": "apphub:ios:123abc", REQUIRED
        "apiHost": "http://localhost:8000",
        "assetDirectory": "example-directory/logrocket", REQUIRED
        "cliPath": "example-path/logrocket", REQUIRED (if logrocket-cli hasn't been added to $PATH)
        "enabledBuildConfigs": ["Debug", "Release"]
      }
    }
  }
*/

const withLogRocket = (config, pluginProps = {}) => {
  // eslint-disable-next-line no-param-reassign
  config = withBuildProperties(config, {
    android: {
      extraMavenRepos: ['https://storage.googleapis.com/logrocket-maven/'],
    },
  });

  if (pluginProps.android && pluginProps.android.assetCapture) {
    // eslint-disable-next-line no-param-reassign
    config = withAndroidLogRocket(config, pluginProps);
  }

  if (pluginProps.ios && pluginProps.ios.assetCapture) {
    // eslint-disable-next-line no-param-reassign
    config = withIosLogRocket(config, pluginProps);
  }

  return config;
};

module.exports = createRunOncePlugin(withLogRocket, 'logrocket');
