#!/usr/bin/env ruby
# frozen_string_literal: true

require 'json'

# Ensure the font data file is freshly generated on each build
app_build_resources_dir = "#{ENV['BUILT_PRODUCTS_DIR']}/#{ENV['UNLOCALIZED_RESOURCES_FOLDER_PATH']}"
output_file = "#{app_build_resources_dir}/lr_custom_fonts_data.json"
if File.exist?(output_file)
  File.delete(output_file)
end


# Check whether LogRocket asset upload is enabled for this build
current_build_config = ENV['CONFIGURATION']
enabled_build_configs = ENV['LOGROCKET_ENABLED_BUILD_CONFIGS']&.split(',')

unless enabled_build_configs&.include?(current_build_config) || current_build_config == 'Release'
  puts "LogRocket custom asset upload is not enabled for this build."
  exit 0
end

# Ensure configuration is set that allows script to find the downloaded LogRocket CLI for execution
LOGROCKET_CLI_PATH = ENV['LOGROCKET_CLI_PATH'] || `which logrocket-cli`.strip
if LOGROCKET_CLI_PATH.empty?
  warn 'Error: LOGROCKET_CLI_PATH is not set and logrocket-cli was not found on $PATH'
  exit 1
end

# Ensure configuration is set to authorize asset upload for your LogRocket application
unless ENV['LOGROCKET_API_KEY']
  warn 'Error: LOGROCKET_API_KEY is not set'
  exit 1
end

# Ensure configuration is set that allows LogRocket CLI to find custom font files
unless ENV['LOGROCKET_ASSET_DIR']
  warn 'Error: LOGROCKET_ASSET_DIR is not set'
  exit 1
end

# Execute asset_upload CLI command and collect output
cli_cmd = "#{LOGROCKET_CLI_PATH} asset_upload #{ENV['LOGROCKET_ASSET_DIR']}"
cli_output = `#{cli_cmd}`


# Parse CLI output to report any errors or generate the build resource used by the LogRocket SDK
urls = []
cli_output.each_line do |line|
  data = begin
    JSON.parse(line)
  rescue StandardError
    nil
  end
  next unless data

  case data['level']
  when 'info'
    if data['url']
      urls << data['url']
    else
      warn "Failed to retrieve cached url for #{data['filepath']}"
    end
  when 'error'
    warn "Error: #{data['message']}"
    exit 1
  else
    warn "Unknown level for LogRocket CLI asset-upload output: #{data['level']}"
    exit 1
  end
end

if urls.empty?
  warn 'No valid custom font assets found for upload'
else
  Dir.mkdir(app_build_resources_dir) unless Dir.exist?(app_build_resources_dir)

  json_data = { urls: urls }.to_json
  File.write(output_file, json_data)
end

exit 0
