import type React from 'react';

type Nullable<T> = {
  [Property in keyof T]: T[Property] | null;
}

declare let LogRocket: ILogRocket;

export = LogRocket;

interface IRequest {
  reqId: string;
  url: string;
  headers: { [key: string]: string | null | undefined };
  body?: string;
  method: string;
  referrer?: string;
  mode?: string;
  credentials?: string;
}

interface IResponse {
  reqId: string;
  status?: number;
  headers: { [key: string]: string | null | undefined };
  body?: string;
  method: string;
  url?: string
}

interface IOptions {
  serverURL?: string,
  enableIPCapture?: boolean,
  uploadIntervalMs?: number,
  viewScanIntervalSeconds?: number,
  logLevel?: string,
  network?: {
    isEnabled?: boolean,
    requestSanitizer?(request: IRequest): null | Nullable<IRequest>,
    responseSanitizer?(response: IResponse): null | Nullable<IResponse>,
  },
  console?: {
    isEnabled?: boolean | {
      log?: boolean,
      info?: boolean,
      debug?: boolean,
      warn?: boolean,
      error?: boolean
    },
    shouldAggregateConsoleErrors?: boolean,
  },
  allowTags?: string[],
  redactionTags?: string[],
  textSanitizer?: 'excluded' | 'none',
  enablePersistence?: boolean,
  connectionType?: 'MOBILE' | 'WIFI',
  iosProxyConfiguration?: IosProxyConfiguration,
  androidProxy?: AndroidProxyConfiguration,
  /** Controls automatic detection of JS errors using Raven.js
   *  Does not impact captureException or aggregation of console errors
   */
  shouldDetectExceptions?: boolean,
  forceCleanStart?: boolean,
  iosThroughputTrackingEnabled?: boolean,
  captureRedactedViewTouches?: boolean,
  disableScrollDelegate?: boolean,
  iosExtendedViewTextCapture?: boolean,
  /** The specified updateId is manually appended to the release version.
   *  This is intended to be used for tracking Expo Update IDs.
   */
  updateId?: string | null,
  /** The specified expoChannel is manually appended to the release version.
   *  This is intended to be used for tracking Expo channels.
   */
  expoChannel?: string | null,
  viewScanningEnabled?: boolean,
  /** Controls whether touch events are captured on iOS */
  registerTouchHandlers?: boolean,
  shouldParseXHRBlob?: boolean,
  /**
   * Control whether automatically registered navigation events
   * for Android activity/iOS view lifecycle changes are disabled.
   * This configuration option does not affect manual page tagging behavior.
   *
   * Defaults to `true`.
   */
  enableAutomaticLifecycleCapture?: boolean,
  /**
   * @deprecated WARNING:
   * Changing this setting may result in highly limited capture or causing the app to hang for extreme lengths of time
   * Proceed at your own risk!
   * Specifies, in milliseconds, how long the LogRocket SDK should attempt to capture a view before timing out to
   * prevent the app from hanging.
   * Defualts to 100 milliseconds.
   */
  viewCaptureTimeoutThreshold?: number,
    /**
   * @deprecated WARNING:
   * Increasing this value may result in the app hangs during view capture
   * Specifies, in milliseconds, how much additional time past viewCaptureTimeoutThreshold the LogRocket SDK should
   * attempt to capture a view using wireframes before timing out to prevent the app from hanging.
   * You may safely disable wireframe capture by setting this config value to 0.
   * Defualts to 20 milliseconds.
   */
  viewCaptureAdditionalWireframeDuration?: number,
  /**
   * Controls whether the LogRocket SDK will automatically capture native iOS logs.
   * Native logs are not captured on Android. This does not affect the capture of console logs.
   * This is enabled by default.
   */
  nativeLogCaptureEnabled?: boolean,
}

type IosProxyConfiguration = {
  proxyUsername?: string,
  proxyPassword?: string,
  httpEnable?: boolean | number,
  httpPort?: number,
  httpProxy?: string,
}

type AndroidProxyConfiguration = {
  port?: number,
  host?: string,
  authHeaderName?: string,
  authHeaderValue?: string,
}

interface IUserTraits {
  [propName: string]: string | number | boolean,
}

type TrackEventProperties = {
  revenue?: number,
  [key: string]: string | number | boolean | string[] | number[] | boolean[] | undefined | null
};

interface IExceptionOptions {
  tags?: {
    [key: string]: string | number | boolean;
  },
  extra?: {
    [key: string]: string | number | boolean;
  },
}

type State = { [key: string]: any };
type Action = { [key: string]: any };

interface IReduxMiddlewareOptions {
  /** Sanitizer function to scrub redux state */
  stateSanitizer?(state: State): State,
  /** Sanitizer function to scrub or ignore specific redux actions */
  actionSanitizer?(action: Action): null | Action,
}

interface ILogRocket {
  init(appID: string, config?: IOptions): void;
  getSessionURL(callback: (sessionURL: string) => void): void;
  /** Identify a user with the current session, with optional user traits */
  identify(uid: string, traits?: IUserTraits): void;
  identify(traits: IUserTraits): void;
  captureException(exception: any, options?: IExceptionOptions): void;
  captureMessage(message: string, options?: IExceptionOptions): void;
  /** Send an event to LogRocket */
  track(eventName: string, eventProperties?: TrackEventProperties): void;
  /** Returns a redux middleware which adds redux logs to LogRocket sessions */
  reduxMiddleware(
    /** Optional sanitizer configuration */
    options?: IReduxMiddlewareOptions
  ): any;
  shutdown(): void;
  pauseViewCapture(): void;
  unpauseViewCapture(): void;
  tagPage(relativePage: string): void;
  /** Register a web view with the SDK in order to link its recording to the current mobile one (Android-specific) */
  registerWebView(view: any): void;
  startNewSession(appID?: string, config?: IOptions): Promise<boolean>;
  startNewSessionSameConfig(): Promise<boolean>;
  log(...args: any[]): void;
  debug(...args: any[]): void;
  info(...args: any[]): void;
  warn(...args: any[]): void;
  error(...args: any[]): void;
}

export function LRAllowProps(allowTag: string = 'lr-show', useNativeID: bool = false);
export const LRAllow: ({ children }: {
  children: React.ReactNode;
}) => React.FunctionComponentElement<RedactionAttributes>;
export const LRNativeIDAllow: ({ children }: {
  children: React.ReactNode;
}) => React.FunctionComponentElement<RedactionAttributes>;


export function LRRedactProps(redactionTag: string = 'lr-hide', useNativeID: bool = false);
export const LRRedact: ({ children }: {
  children: React.ReactNode;
}) => React.FunctionComponentElement<RedactionAttributes>;
export const LRNativeIDRedact: ({ children }: {
  children: React.ReactNode;
}) => React.FunctionComponentElement<RedactionAttributes>;
